==> MuSE-0.2.tar.gz: MuSE-0.2/netlistener.cpp <==
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <stream.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>

#include "jmixer.h"
#include "libsync/synchronous.h"

#define BUFFERSIZE 256
#define PASSWD "hackme"
#define HELLO "MuSE daemon v0.2.0\n\n"

class Net_listener: public LTI::Object {

private:
  int num_porta;
  int id_socket;

  bool disconnect;
  bool quit;
  char buffer[BUFFERSIZE];
  struct hostent *phost;
  struct timeval timeout;
  struct sockaddr_in nome_socket;
  struct sockaddr_in nome_accept;
  unsigned int dim_nome_accept;
  Stream_mixer *mixer;

public:
  Net_listener(int porta, Stream_mixer *mix);

  void run();
  int esegui_comando(char *buf);
  void send_playlist();
  int accetta_socket();
  void chiudi_socket();
  void get_connection();
  int getin();
  int getout(const char *buf);

  int id_accept;
  
  ~Net_listener();
};

Net_listener::Net_listener(int porta, Stream_mixer *mix) {
  num_porta = porta;
  int one = 1;
  
  synchronous.registerObjectName("NetListener");
  
  if((id_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_IP)) < 0)
    perror("Unable to open tcp/ip socket");
  
  if(setsockopt(id_socket, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(int)) < 0)
    perror("Unable to set socket parameters");
  
  memset(&nome_socket,0,sizeof(struct sockaddr_in));
  memset(&nome_accept,0,sizeof(struct sockaddr_in));
  
  nome_socket.sin_family = AF_INET;
  nome_socket.sin_port = htons(num_porta);
  nome_socket.sin_addr.s_addr = htonl(INADDR_ANY);
  
  id_accept = -2;
  
  if(bind(id_socket, (struct sockaddr *) &nome_socket, sizeof(nome_socket))==-1)
    perror("Unable to bind tcp/ip connection");
  
  if(listen(id_socket, 5)==-1)
    perror("Cannot listen over tcp/ip");

  mixer = mix;

  disconnect = false;
  quit = false;

  mixer->client = &id_accept;

}

void Net_listener::get_connection() {
  bool authorized = false;
  while(!authorized) {
    dim_nome_accept = sizeof(struct sockaddr_in);
    id_accept = accept(id_socket, (struct sockaddr *) &nome_accept, &dim_nome_accept);

    getout("PWD\n\n");
    getin();
    if(strncmp(PASSWD,buffer,strlen(PASSWD))==0)
      authorized = true;
    else {
      getout("Sorry, authentication failed\n");
      chiudi_socket();
    }
  }
  getout(HELLO);
}    

void Net_listener::run() {
  int res;
  
  if(debug)
    cerr << selfName() << " running on thread " << (int)pthread_self() << "\n";
  
  while(!quit) {
    get_connection();
    send_playlist();
    do {
      if(getin()<=0)
	break;
      buffer[3] = '\0';
      res = esegui_comando(buffer);
      switch(res) {
      case 0:
	getout("400\n\n"); /* ok, request shall have success */
	break;
      case 1:
	getout("401\n\n"); /* syntax incorrect */
	  break;
      case 2:
	getout("402\n\n"); /* channel in use */
	break;
      case 3:
	getout("403\n\n"); /* channel not allocated */
	break;
      case 4:
	getout("404\n\n"); /* the request cannot be processed */
	break;
      case 5:
	getout("405\n"); /* playlist position out of bounds */
	break;
      }
      
    } while(!disconnect);
    chiudi_socket();
    disconnect = false;
  } /* while(!quit) */
  
  cerr << selfName() << " exiting " << "\n";
  
}

int Net_listener::esegui_comando(char *buf) {

  char *p, *pp;
 
  /* 301: start mp3 file on channel with volume & speed
     301'\t'(int)fl'\t'(int)ch'\t'(float)speed'\t'(float)volume'\n' */
    if(strncmp(buf,"301",3)==0) {
      int fnum, ch;
      float speed, vol;
      p = pp = &buf[4];
      while(*p!='\t') p++; *p = '\0';
      fnum = atoi(pp);
      p++; pp = p;
      while(*p!='\t') p++; *p = '\0';
      ch = atoi(pp);
      p++; pp = p;
      while(*p!='\t') p++; *p = '\0';
      speed = atof(pp);
      p++; pp = p;
      while(*p!='\n') p++; *p = '\0';
      vol = atof(pp);
      
      fprintf(stderr,"cmd:301 file:%u channel:%u speed:%f volume:%f\n",ch,fnum,speed,vol);
      
      synchronous.select.call(0, mixer, Stream_mixer::id_set_channel);
      synchronous.wait();
      mixer->set_channel(ch,fnum);
      
      if(vol!=1.0) {
	synchronous.select.call(0, mixer, Stream_mixer::id_set_volume);
	synchronous.wait();
	mixer->set_volume(ch,vol);
      }
      
      if(speed!=1.0) {
	synchronous.select.call(0, mixer, Stream_mixer::id_set_speed);
	synchronous.wait();
	mixer->set_speed(ch,speed);
      }
      return(0);
    }
    
    /* 302: stop channel - frees the channel and stops the stream
       302'\t'(int)ch'\n' */
    if(strncmp(buf,"302",3)==0) {
      int ch;
      p = pp = &buf[4];
      while(*p!='\n') p++; *p = '\0';
      ch = atoi(pp);

      fprintf(stderr,"cmd:302 channel:%u\n",ch);

      synchronous.select.call(0, mixer, Stream_mixer::id_stop_channel);
      synchronous.wait();
      mixer->stop_channel(ch);
      
      return(0);
    }

    /* 303: pause channel - stop reading from the channel stream (without kill)
       303'\t'(int)ch'\n' */
    if(strncmp(buf,"303",3)==0) {
      int ch;
      p = pp = &buf[4];
      while(*p!='\n') p++; *p = '\0';
      ch = atoi(pp);

      fprintf(stderr,"cmd:303 channel:%u\n",ch);

      synchronous.select.call(0, mixer, Stream_mixer::id_pause_channel);
      synchronous.wait();
      mixer->pause_channel(ch);
      
      return(0);
    }

    /* 304: start channel - cannot be used after killing, shall be use after pause
       304'\t'(int)ch'\n' */
    if(strncmp(buf,"304",3)==0) {
      int ch;
      p = pp = &buf[4];
      while(*p!='\n') p++; *p = '\0';
      ch = atoi(pp);
      
      fprintf(stderr,"cmd:304 channel:%u\n",ch);

      synchronous.select.call(0, mixer, Stream_mixer::id_start_channel);
      synchronous.wait();
      mixer->start_channel(ch);
      
      return(0);
    }

    /* 305: set speed - change the speed of a channel
       305'\t'(int)ch'\t'(float)speed'\n' */
    if(strncmp(buf,"305",3)==0) {
      int ch;
      float speed;
      p = pp = &buf[4];
      while(*p!='\t') p++; *p = '\0';
      ch = atoi(pp);
      p++; pp = p;
      while(*p!='\n') p++; *p = '\0';
      speed = atof(pp);
      
      fprintf(stderr,"cmd:305 channel:%u speed:%f\n",ch,speed);

      synchronous.select.call(0, mixer, Stream_mixer::id_set_speed);
      synchronous.wait();
      mixer->set_speed(ch,speed);

      return(0);
    }

    /* 306: set volume - change the volume of a channel
       306:'\t'(int)ch'\t'(float)volume'\n' */
    if(strncmp(buf,"306",3)==0) {
      int ch;
      float vol;
      p = pp = &buf[4];
      while(*p!='\t') p++; *p = '\0';
      ch = atoi(pp);
      p++; pp = p;
      while(*p!='\n') p++; *p = '\0';
      vol = atof(pp);
      
      fprintf(stderr,"cmd:306 channel:%u volume:%f\n",ch,vol);

      synchronous.select.call(0, mixer, Stream_mixer::id_set_volume);
      synchronous.wait();
      mixer->set_volume(ch,vol);

      return(0);
    }
    
    /* 307: set live - switch live input channel on/off
       307'\t'(int)num'\n' */
    if(strncmp(buf,"307",3)==0) {
      int stat;
      p = pp = &buf[4];
      while(*p!='\n') p++; *p = '\0';
      stat = atoi(pp);

      fprintf(stderr,"cmd:307 status:%u\n",stat);

      synchronous.select.call(0,mixer, Stream_mixer::id_set_live);
      synchronous.wait();
      mixer->set_live(stat);

      return(0);
    }
    
    /* 333: dump status - dump channel status on MuSE stderr
       333'\n' */
    if(strncmp(buf,"333",3)==0) {
      synchronous.select.call(0, mixer, Stream_mixer::id_dump_status);
      synchronous.wait();
      mixer->dump_status();
      return(0);
    }

    /* 666: close connection and kill muse
       666'\n' */
    if(strncmp(buf,"666",3)==0) {
      disconnect = true;
      return(0);
    }

    return(1); /* syntax incorrect */
}

void Net_listener::send_playlist() {
  char temp[MAX_PATH_SIZE+2];
  int c = 0;
  
  getout("201\n\n");

  while(mixer->playlist[c][0]!='\0') {
    sprintf(temp,"%s\n\n",&mixer->playlist[c][0]);
    getout(temp);
    c++;
  }

  getout("202\n\n");
}

void Net_listener::chiudi_socket() {
  close(id_accept);
  id_accept = -2;
}

int Net_listener::getout(const char *buf) {
  return(send(id_accept, buf, strlen(buf+1), 0x0));
}

int Net_listener::getin() {
  int c;

  c = recv(id_accept, buffer, BUFFERSIZE, 0x0);

  if (c<0) return(c);

  /* make it terminate with a \0 */
  if(c<BUFFERSIZE)
    buffer[c] = '\0';
  else
    /* we have a buffer overflow here */
    cerr << "!! Net_listener's buffer overflowed by a command request\n";

  return(c);
}

Net_listener::~Net_listener() {
  if(id_accept > -1)
    chiudi_socket();
  if(buffer==NULL)
    free(buffer);
}

==> MuSE-0.3.tar.gz: MuSE-0.3/netlistener.cpp <==
#include <iostream.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <stream.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>

#include "jmixer.h"
#include "libsync/synchronous.h"

#define BUFFERSIZE 256
#define PASSWD "hackme"
#define HELLO "MuSE daemon v0.3.0\n\n"

class Net_listener: public LTI::Object {

private:
  int num_porta;
  int id_socket;

  bool disconnect;

  char buffer[BUFFERSIZE];
  struct hostent *phost;
  struct timeval timeout;
  struct sockaddr_in nome_socket;
  struct sockaddr_in nome_accept;
  unsigned int dim_nome_accept;
  Stream_mixer *mixer;

public:
  Net_listener(int porta, Stream_mixer *mix);

  void run();
  int esegui_comando(char *buf);
  void send_playlist();
  int accetta_socket();
  void chiudi_socket();
  void get_connection();
  int getin();
  int getout(const char *buf);

  int id_accept;
  
  ~Net_listener();
};

Net_listener::Net_listener(int porta, Stream_mixer *mix) {
  num_porta = porta;
  int one = 1;
  
  synchronous.registerObjectName("NetListener");
  
  if((id_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_IP)) < 0)
    perror("Unable to open tcp/ip socket");
  
  if(setsockopt(id_socket, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(int)) < 0)
    perror("Unable to set socket parameters");
  
  memset(&nome_socket,0,sizeof(struct sockaddr_in));
  memset(&nome_accept,0,sizeof(struct sockaddr_in));
  
  nome_socket.sin_family = AF_INET;
  nome_socket.sin_port = htons(num_porta);
  nome_socket.sin_addr.s_addr = htonl(INADDR_ANY);
  
  id_accept = -2;
  
  if(bind(id_socket, (struct sockaddr *) &nome_socket, sizeof(nome_socket))==-1)
    perror("Unable to bind tcp/ip connection");
  
  if(listen(id_socket, 5)==-1)
    perror("Cannot listen over tcp/ip");

  mixer = mix;

  disconnect = false;

  mixer->client = &id_accept;

}

void Net_listener::get_connection() {
  bool authorized = false;
  while(!authorized) {
    dim_nome_accept = sizeof(struct sockaddr_in);
    id_accept = accept(id_socket, (struct sockaddr *) &nome_accept, &dim_nome_accept);

    getout("PWD\n");
    getin();

    if(strncmp(PASSWD,buffer,strlen(PASSWD))==0)
      authorized = true;
    else {
      getout("Sorry, authentication failed\n");
      chiudi_socket();
    }
  }
  getout(HELLO);

  /* dirty dirty dirty dirty dirty dirty dirty dirty dirty dirty
     but effective right now.
     in fact we're going to make a playlist selector soon.... 'nuff said */
  getout("201\n");
  int c;
  for(c=0;c<mixer->mp3ch[0].playlist_len;c++) {
    getout(mixer->mp3ch[0].playlist[c]);
    getout("\n");
  }
  getout("202\n");

  disconnect = false;

}    

void Net_listener::run() {
  int res;
  
  if(debug)
    cerr << selfName() << " running on thread " << (int)pthread_self() << "\n";
  
  while(!mixer->quit) {
    get_connection();
    /* send_playlist(); */
    do {
      if(getin()<=0)
	break;
      buffer[3] = '\0';
      res = esegui_comando(buffer);
      switch(res) {
      case 0:
	getout("400\n"); /* ok, request shall have success */
	break;
      case 1:
	getout("401\n"); /* syntax incorrect */
	break;
      case 2:
	getout("402\n"); /* problem connecting to icecast */
	break;
      default:
	getout("403\n"); /* i got a bad feeling! */
	break;
	/*
	  case 2:
	  getout("402\n\n"); // channel in use 
	  break;
	  case 3:
	  getout("403\n\n"); // channel not allocated
	  break;
	  case 4:
	  getout("404\n\n"); // the request cannot be processed
	  break;
	  case 5:
	  getout("405\n"); // playlist position out of bounds
	  break;
	*/
      }
      
    } while(!disconnect);
    chiudi_socket();
  } /* while(!mixer->quit) */
  
  cerr << selfName() << " exiting " << "\n";
  
}

int Net_listener::esegui_comando(char *buf) {

  char *p, *pp;
 
  /* 301: start mp3 file on channel with volume & speed
     301'\t'(int)fl'\t'(int)ch'\t'(float)speed'\t'(float)volume'\n' */
    if(strncmp(buf,"301",3)==0) {
      int fnum, ch;
      float speed, vol;
      p = pp = &buf[4];
      while(*p!='\t') p++; *p = '\0';
      fnum = atoi(pp);
      p++; pp = p;
      while(*p!='\t') p++; *p = '\0';
      ch = atoi(pp);
      p++; pp = p;
      while(*p!='\t') p++; *p = '\0';
      speed = atof(pp);
      p++; pp = p;
      while(*p!='\n') p++; *p = '\0';
      vol = atof(pp);
      
      fprintf(stderr,"cmd:301 file:%u channel:%u speed:%f volume:%f\n",ch,fnum,speed,vol);
      
      synchronous.select.call(0, mixer, Stream_mixer::id_set_channel);
      synchronous.wait();
      mixer->set_channel(ch,fnum);
      
      if(vol!=1.0) {
	synchronous.select.call(0, mixer, Stream_mixer::id_set_volume);
	synchronous.wait();
	mixer->set_volume(ch,vol);
      }
      
      if(speed!=1.0) {
	synchronous.select.call(0, mixer, Stream_mixer::id_set_speed);
	synchronous.wait();
	mixer->set_speed(ch,speed);
      }
      return(0);
    }
    
    /* 302: stop channel - frees the channel and stops the stream
       302'\t'(int)ch'\n' */
    if(strncmp(buf,"302",3)==0) {
      int ch;
      p = pp = &buf[4];
      while(*p!='\n') p++; *p = '\0';
      ch = atoi(pp);

      fprintf(stderr,"cmd:302 channel:%u\n",ch);

      synchronous.select.call(0, mixer, Stream_mixer::id_stop_channel);
      synchronous.wait();
      mixer->stop_channel(ch);
      
      return(0);
    }

    /* 303: pause channel - stop reading from the channel stream (without kill)
       303'\t'(int)ch'\n' */
    if(strncmp(buf,"303",3)==0) {
      int ch;
      p = pp = &buf[4];
      while(*p!='\n') p++; *p = '\0';
      ch = atoi(pp);

      fprintf(stderr,"cmd:303 channel:%u\n",ch);

      synchronous.select.call(0, mixer, Stream_mixer::id_pause_channel);
      synchronous.wait();
      mixer->pause_channel(ch);
      
      return(0);
    }

    /* 304: resume channel - cannot be used after killing, shall be use after pause
       304'\t'(int)ch'\n' */
    if(strncmp(buf,"304",3)==0) {
      int ch;
      p = pp = &buf[4];
      while(*p!='\n') p++; *p = '\0';
      ch = atoi(pp);
      
      fprintf(stderr,"cmd:304 channel:%u\n",ch);

      synchronous.select.call(0, mixer, Stream_mixer::id_resume_channel);
      synchronous.wait();
      mixer->resume_channel(ch);
      
      return(0);
    }

    /* 305: set speed - change the speed of a channel
       305'\t'(int)ch'\t'(float)speed'\n' */
    if(strncmp(buf,"305",3)==0) {
      int ch;
      float speed;
      p = pp = &buf[4];
      while(*p!='\t') p++; *p = '\0';
      ch = atoi(pp);
      p++; pp = p;
      while(*p!='\n') p++; *p = '\0';
      speed = atof(pp);
      
      fprintf(stderr,"cmd:305 channel:%u speed:%f\n",ch,speed);

      synchronous.select.call(0, mixer, Stream_mixer::id_set_speed);
      synchronous.wait();
      mixer->set_speed(ch,speed);

      return(0);
    }

    /* 306: set volume - change the volume of a channel
       306:'\t'(int)ch'\t'(float)volume'\n' */
    if(strncmp(buf,"306",3)==0) {
      int ch;
      float vol;
      p = pp = &buf[4];
      while(*p!='\t') p++; *p = '\0';
      ch = atoi(pp);
      p++; pp = p;
      while(*p!='\n') p++; *p = '\0';
      vol = atof(pp);
      
      fprintf(stderr,"cmd:306 channel:%u volume:%f\n",ch,vol);

      synchronous.select.call(0, mixer, Stream_mixer::id_set_volume);
      synchronous.wait();
      mixer->set_volume(ch,vol);

      return(0);
    }
    
    /* 307: set live - switch live input channel on/off
       307'\t'(int)num'\n' */
    if(strncmp(buf,"307",3)==0) {
      int stat;
      p = pp = &buf[4];
      while(*p!='\n') p++; *p = '\0';
      stat = atoi(pp);

      fprintf(stderr,"cmd:307 status:%u\n",stat);

      synchronous.select.call(0,mixer, Stream_mixer::id_set_live);
      synchronous.wait();
      if(mixer->set_live(stat))
	return(0);

      return(4);
    }

    /* 308: set dsp out - RESERVED */

    /* 309: set playlist - set/refreshes a playlist for a channel
       309'\t'(int)ch'\t'(char)playlist'\n'
       (char)playlist is the significative part of the standard playlist path
       i.e. .muse/*default* */
    if(strncmp(buf,"309",3)==0) {
      int ch;
      p = pp = &buf[4];
      while(*p!='\t') p++; *p = '\0';
      ch = atoi(pp);
      p++; pp = p;
      while(*p!='\n') p++; *p = '\0';
      
      synchronous.select.call(0, mixer, Stream_mixer::id_set_playlist);
      synchronous.wait();
      mixer->set_playlist(ch, pp);
      return(0);
    }

    /* 310: connect to icecast server
       310'\t'ip address'\t'port'\t'mount'\t'pass'\t'bitrate'\n' */
    if(strncmp(buf,"310",3)==0) {
      char ip[20];
      int port;
      char pass[64];
      char mountpoint[64];
      int bps;

      p = pp = &buf[4];
      while(*p!='\t') p++; *p = '\0';
      strcpy(ip,pp);
      p++; pp = p;
      while(*p!='\t') p++; *p = '\0';
      port = atoi(pp);
      p++; pp = p;
      while(*p!='\t') p++; *p = '\0';
      strcpy(mountpoint,pp);
      p++; pp = p;
      while(*p!='\t') p++; *p = '\0';
      strcpy(pass,pp);
      p++; pp = p;
      while(*p!='\n') p++; *p = '\0';
      bps = atoi(pp);

      synchronous.select.call(0, mixer, Stream_mixer::id_set_icecast);
      synchronous.wait();
      if(mixer->set_icecast(ip,port,pass,mountpoint,bps))
	return(0);
      else
	return(2);
    }
    
    /* 311: disconnect from icecast server
       311'\n' */
    if(strncmp(buf,"311",3)==0) {
      synchronous.select.call(0, mixer, Stream_mixer::id_stop_icecast);
      synchronous.wait();
      mixer->stop_icecast();
      return(0);
    }
    
    
    /* 333: dump status - dump channel status on MuSE stderr
       333'\n' */
    if(strncmp(buf,"333",3)==0) {
      synchronous.select.call(0, mixer, Stream_mixer::id_dump_status);
      synchronous.wait();
      mixer->dump_status();
      return(0);
    }

    /* 666: close connection but leave muse running accepting other connections
       666'\n' */
    if(strncmp(buf,"666",3)==0) {
      disconnect = true;
      return(0);
    }

    /* 999: close connection and stop muse
       999'\n' */
    if(strncmp(buf,"999",3)==0) {
      disconnect = true;
      mixer->quit = true;
      return(0);
    }

    return(1); /* syntax incorrect */
}

void Net_listener::send_playlist() {
  char temp[MAX_PATH_SIZE+2];
  int c = 0;
  int cc;
  
  for(cc=0;cc<MAX_CHANNELS;cc++) {
    sprintf(temp,"201\t%u\n",cc);
    getout(temp);
    while(mixer->mp3ch[cc].playlist[c][0]!='\0') {
      sprintf(temp,"%s\n",&mixer->mp3ch[cc].playlist[c][0]);
      getout(temp);
      c++;
    }
  }

  getout("202\n");
}

void Net_listener::chiudi_socket() {
  close(id_accept);
  id_accept = -2;
}

int Net_listener::getout(const char *buf) {
  return(send(id_accept, buf, strlen(buf), 0x0));
}

int Net_listener::getin() {
  int c;

  c = recv(id_accept, buffer, BUFFERSIZE, 0x0);

  if (c<0) return(c);

  /* make it terminate with a \0 */
  if(c<BUFFERSIZE)
    buffer[c] = '\0';
  else
    /* we have a buffer overflow here */
    cerr << "!! Net_listener's buffer overflowed by a command request\n";

  return(c);
}

Net_listener::~Net_listener() {
  if(id_accept > -1)
    chiudi_socket();
  if(buffer==NULL)
    free(buffer);
}
